require 'set'

class RedisSearch
  attr_reader :redis, :namespace

  # Public: Instantiate a new RedisSearch instance.
  #
  # opts - A Hash of options.
  #        :redis     - A Redis instance.
  #        :namespace - The String namespace for Redis keys.
  #
  # Returns nothing.
  def initialize(opts = {})
    @redis = opts[:redis]
    @namespace = opts[:namespace]
  end

  # Public: Index a document.
  #
  # id  - The ID of the document.
  # doc - The String document to index.
  #
  # Returns nothing.
  def index(id, doc)
    redis.pipelined do
      tokens(doc).uniq.each do |token|
        redis.lpush key(token), id
      end
    end
  end

  # Public: Search for a document.
  #
  # query - The String query.
  #
  # Returns an Array of matching document IDs.
  def search(query)
    redis.pipelined do
      tokens(query).uniq.each do |token|
        redis.lrange(key(token), -100, -1)
      end
    end.flatten.uniq.map do |res|
      res.force_encoding('ascii-8bit')
    end
  end

  private
  # Private: Namespace a Redis key.
  #
  # token - The token to namespace.
  #
  # Returns a String.
  def key(token)
    [namespace, token].compact.join(':')
  end

  # Private: Tokenize a document or query.
  #
  # string - The String to tokenize.
  #
  # Returns an Array of String tokens.
  def tokens(string)
    string.scan(/\w+/)
  end
end
